@@ -1,22 +1,35 @@
 """
-Update release numbers in various places, according to a release.ini file places at the project root
+Update release numbers in various places, according to a release.ini file places at the project root.
+
+For now, the following sections are supported:
+
++ Main file (__init__.py, Django settings file, etc.)
++ Sphinx conf.py file
++ sonar-project.properties
++ ansible vars file
++ node package.json file
++ setup.cfg
++ setup.py
++ pyproject.toml
+
 """
 import configparser
 import logging
 import sys
 from configparser import ConfigParser
 from pathlib import Path
-from typing import Optional, Tuple
+from typing import Optional
 
 import click
 
 from bump_release import helpers
-from bump_release.helpers import split_version
 
 # region Globals
-__version__ = VERSION = "0.9.6"
+__version__ = VERSION = "0.9.10"
 RELEASE_FILE: Optional[Path] = None
 RELEASE_CONFIG: Optional[ConfigParser] = None
+
+
 # endregion Globals
 
 
@@ -26,6 +39,8 @@ RELEASE_CONFIG: Optional[ConfigParser] = None
     "--release-file",
     "release_file",
     help="Release file path, default `./release.ini`",
+    type=click.Path(exists=True),
+    default="release.ini",
 )
 @click.option(
     "-n",
@@ -52,13 +67,27 @@ def bump_release(
     debug: bool = False,
 ) -> int:
     """
-    Updates the files according to the release.ini file
+    Update release numbers in various places, according to a release.ini file places at the project root.
 
-    :param release: Version number, as "X.X.X"
-    :param release_file: path to the release.ini config file
+    \b
+    For now, the following sections are supported:
+
+    \b
+    + Main file (__init__.py, Django settings file, etc.)
+    + Sphinx conf.py file
+    + sonar-project.properties
+    + ansible vars file
+    + node package.json file
+    + setup.cfg
+    + setup.py
+    + pyproject.toml (Require using `pip install bump_release[toml]`)
+
+    \f
+    :param release: Release number
+    :param release_file: Release file path, default `./release.ini`
     :param dry_run: If `True`, no operation performed
-    :param debug: If `True`, more traces !
-    :return: 0 if no error...
+    :param debug: If `True`, more traces are printed for users
+    :return: 0 if success, 1|2 if error
     """
     # Loads the release.ini file
     global RELEASE_CONFIG, RELEASE_FILE
@@ -81,7 +110,16 @@ def bump_release(
 
 
 def process_update(release_file: Path, release: str, dry_run: bool, debug: bool = False) -> int:
-    version = split_version(release)
+    """
+    Processes the update of the release number.
+
+    :param release_file: Release file path
+    :param release: Release number
+    :param dry_run: If `True`, no operation performed
+    :param debug: If `True`, more traces are printed for users
+    :return: 0 if success, 1|2 if error
+    """
+    version = helpers.VersionNumber(version=release)
 
     # Initialize the logging
     if debug:
@@ -116,6 +154,15 @@ def process_update(release_file: Path, release: str, dry_run: bool, debug: bool
         logging.warning(f"process_update() No release section for `setup`: {e}")
     # endregion
 
+    # region Update setup.cfg file
+    try:
+        new_row = update_setup_cfg_file(version=version, dry_run=dry_run)
+        if new_row is not None:
+            logging.debug(f"process_update() `setup`: new_row = {new_row.strip()}")
+    except helpers.NothingToDoException as e:
+        logging.warning(f"process_update() No release section for `setup`: {e}")
+    # endregion Update setup.cfg file
+
     # region Updates sphinx file
     try:
         new_row = update_docs_conf(version=version, dry_run=dry_run)
@@ -151,12 +198,25 @@ def process_update(release_file: Path, release: str, dry_run: bool, debug: bool
         logging.warning(f"process_update() `release.ini`: new_row = {new_row.strip()}")
     # endregion
 
+    # region Updates pyproject.toml file
+    try:
+        new_row = update_pyproject(version=version, dry_run=dry_run)
+        if new_row is not None:
+            logging.debug(f"process_update() `pyproject.toml`: new_row = {new_row.strip()}")
+    except helpers.NothingToDoException as e:
+        logging.warning(f"process_update() No release section for `pyproject.toml`: {e}")
+    except helpers.UpdateException:  # noqa: E722
+        logging.exception("process_update() `pyproject.toml`: UpdateException - tomli / tomli_w might be missing")
+        return 4
+
+    # endregion Updates pyproject.toml file
+
     return 0
 
 
-def update_main_file(version: Tuple[str, str, str], dry_run: bool = True) -> Optional[str]:
+def update_main_file(version: helpers.VersionNumber, dry_run: bool = True) -> Optional[str]:
     """
-    Updates the main django settings file, or a python script with
+    Updates the main django settings file, or a python script with a __init__.py file.
 
     :param version: Release number tuple (major, minor, release)
     :param dry_run: If `True`, no operation performed
@@ -178,9 +238,9 @@ def update_main_file(version: Tuple[str, str, str], dry_run: bool = True) -> Opt
     return helpers.update_file(path=path, pattern=pattern, template=template, version=version, dry_run=dry_run)
 
 
-def update_setup_file(version: Tuple[str, str, str], dry_run: bool = False) -> Optional[str]:
+def update_setup_file(version: helpers.VersionNumber, dry_run: bool = False) -> Optional[str]:
     """
-    Updates the setup.py file
+    Updates the setup.py file.
 
     :param version: Release number tuple (major, minor, release)
     :param dry_run: If `True`, no operation performed
@@ -201,7 +261,30 @@ def update_setup_file(version: Tuple[str, str, str], dry_run: bool = False) -> O
     return helpers.update_file(path=path, pattern=pattern, template=template, version=version, dry_run=dry_run)
 
 
-def update_sonar_properties(version: Tuple[str, str, str], dry_run: bool = False) -> Optional[str]:
+def update_setup_cfg_file(version: helpers.VersionNumber, dry_run: bool = False) -> Optional[str]:
+    """
+    Update the setup.cfg file.
+
+    :param version: Release number tuple (major, minor, release)
+    :param dry_run: If `True`, no operation performed
+    :return: changed string
+    """
+    assert RELEASE_CONFIG is not None
+    if not RELEASE_CONFIG.has_section("setup_cfg"):
+        raise helpers.NothingToDoException("No `setup_cfg` section in release.ini file")
+
+    try:
+        _path = RELEASE_CONFIG["setup_cfg"].get("path")
+        path = Path(_path)
+        pattern = RELEASE_CONFIG["setup_cfg"].get("pattern", "").strip('"') or helpers.SETUP_CFG_PATTERN
+        template = RELEASE_CONFIG["setup_cfg"].get("template", "").strip('"') or helpers.SETUP_CFG_TEMPLATE
+
+    except configparser.Error as e:
+        raise helpers.NothingToDoException("No action to perform for setup.cfg file", e)
+    return helpers.update_file(path=path, pattern=pattern, template=template, version=version, dry_run=dry_run)
+
+
+def update_sonar_properties(version: helpers.VersionNumber, dry_run: bool = False) -> Optional[str]:
     """
     Updates the sonar-project.properties file with the new release number
 
@@ -223,7 +306,7 @@ def update_sonar_properties(version: Tuple[str, str, str], dry_run: bool = False
     return helpers.update_file(path=path, pattern=pattern, template=template, version=version, dry_run=dry_run)
 
 
-def update_docs_conf(version: Tuple[str, str, str], dry_run: bool = False) -> Optional[str]:
+def update_docs_conf(version: helpers.VersionNumber, dry_run: bool = False) -> Optional[str]:
     """
     Updates the Sphinx conf.py file with the new release number
 
@@ -264,7 +347,7 @@ def update_docs_conf(version: Tuple[str, str, str], dry_run: bool = False) -> Op
     return str(update_release) + str(update_version)
 
 
-def update_node_package(version: Tuple[str, str, str], dry_run: bool = False) -> Optional[str]:
+def update_node_package(version: helpers.VersionNumber, dry_run: bool = False) -> Optional[str]:
     """
     Updates the nodejs package file with the new release number
 
@@ -281,7 +364,7 @@ def update_node_package(version: Tuple[str, str, str], dry_run: bool = False) ->
     return helpers.update_node_packages(path=path, version=version, key=key, dry_run=dry_run)
 
 
-def update_ansible_vars(version: Tuple[str, str, str], dry_run: bool = False) -> Optional[str]:
+def update_ansible_vars(version: helpers.VersionNumber, dry_run: bool = False) -> Optional[str]:
     """
     Updates the ansible project variables file with the new release number
 
@@ -298,7 +381,7 @@ def update_ansible_vars(version: Tuple[str, str, str], dry_run: bool = False) ->
     return helpers.updates_yaml_file(path=path, version=version, key=key, dry_run=dry_run)
 
 
-def update_release_ini(path: Path, version: Tuple[str, str, str], dry_run: bool = False) -> Optional[str]:
+def update_release_ini(path: Path, version: helpers.VersionNumber, dry_run: bool = False) -> Optional[str]:
     """
     Updates the release.ini file with the new release number
 
@@ -314,3 +397,29 @@ def update_release_ini(path: Path, version: Tuple[str, str, str], dry_run: bool
         version=version,
         dry_run=dry_run,
     )
+
+
+def update_pyproject(version: helpers.VersionNumber, dry_run: bool = False) -> Optional[str]:
+    """
+    Updates the pyproject.toml file with the new release number
+
+    :param version: release number, as (<major>, <minor>, <release>)
+    :param dry_run: If `True`, the operation WILL NOT be performed
+    :return: Updated lines
+    """
+    assert RELEASE_CONFIG is not None
+    try:
+        path = Path(RELEASE_CONFIG.get("pyproject", "path"))
+        key = RELEASE_CONFIG.get("pyproject", "key", fallback=helpers.PYPROJECT_KEY)  # noqa
+    except KeyError as ke:
+        raise helpers.NothingToDoException("No pyproject key found in ini file for pyproject.toml file", ke)
+    except configparser.Error as e:
+        raise helpers.NothingToDoException("No action to perform for pyproject.toml file", e)
+
+    # Update pyproject.toml file
+    return helpers.update_toml_file(
+        path=path,
+        version=version,
+        key=key,
+        dry_run=dry_run,
+    )
